from ..jsonrpc.http_net import *


def version():
    r = net_version()
    return r['result']


def peerCount():
    r = net_peerCount()
    number = hex2int(r['result'])
    return number


def listening():
    r = net_listening()
    return r['result']


from .utl import *
from ._http import *


@traxa_rpc
def taraxa_syncing():
    params = []
    return params


@traxa_rpc
def taraxa_coinbase(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_mining():
    params = []
    return params


@traxa_rpc
def taraxa_hashrate(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_gasPrice(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_accounts(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getBalance(address, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


@traxa_rpc
def taraxa_blockNumber(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getStorageAt(address, position, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, position, tag]
    return params


@traxa_rpc
def taraxa_getTransactionCount(address, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


@traxa_rpc
def taraxa_getBlockTransactionCountByHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def taraxa_getBlockTransactionCountByNumber(tag, **kwargs):
    tag = tag_check(tag)
    params = [tag]
    return params


@traxa_rpc
def taraxa_getUncleCountByBlockHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def taraxa_getUncleCountByBlockNumber(tag, **kwargs):
    tag = tag_check(tag)
    params = [tag]
    return params


@traxa_rpc
def taraxa_getCode(address, tag, **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


# TODO
@traxa_rpc
def taraxa_sign(address, data, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, data]
    return params


# TODO
@traxa_rpc
def taraxa_sendTransaction(trx, **kwargs):
    params = [trx]
    return params


@traxa_rpc
def taraxa_sendRawTransaction(trx, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [trx]
    return params


# TODO
@traxa_rpc
def taraxa_call(trx, tag, **kwargs):
    tag = tag_check(tag)
    params = [
        {
            # "from": trx['from'],
            # "to": trx['to'],
            # "gas": trx['gas'],
            # "gasPrice": trx['gasPrice'],
            # "value": trx['value'],
            # "data": trx['data'],
        },
        tag
    ]
    return params


# TODO
@traxa_rpc
def taraxa_estimateGas(trx, tag, **kwargs):
    tag = tag_check(tag)
    params = [
        {
            # "from": trx['from'],
            # "to": trx['to'],
            # "gas": trx['gas'],
            # "gasPrice": trx['gasPrice'],
            # "value": trx['value'],
            # "data": trx['data'],
        },
        tag
    ]
    return params


@traxa_rpc
def taraxa_getBlockByHash(hash, fullTransactions=False, **kwargs):
    params = [hash, fullTransactions]
    return params


@traxa_rpc
def taraxa_getBlockByNumber(tag, fullTransactions=False, **kwargs):
    tag = tag_check(tag)
    params = [tag, fullTransactions]
    return params


@traxa_rpc
def taraxa_getTransactionByHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def taraxa_getTransactionByBlockHashAndIndex(hash, index, **kwargs):
    params = [hash, index]
    return params


@traxa_rpc
def taraxa_getTransactionByBlockNumberAndIndex(tag, index, **kwargs):
    tag = tag_check(tag)
    params = [tag, index]
    return params


@traxa_rpc
def taraxa_getTransactionReceipt(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def taraxa_pendingTransactions(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getUncleByBlockHashAndIndex(hash, index, **kwargs):
    params = [hash, index]
    return params


@traxa_rpc
def taraxa_getUncleByBlockNumberAndIndex(tag, index, **kwargs):
    tag = tag_check(tag)
    params = [tag, index]
    return params


@traxa_rpc
def taraxa_newFilter(filter, **kwargs):
    params = [filter]
    return params


@traxa_rpc
def taraxa_newBlockFilter(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_newPendingTransactionFilter(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_uninstallFilter(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def taraxa_getFilterChanges(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def taraxa_getFilterLogs(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def taraxa_getLogs(filter, **kwargs):
    params = [filter]
    return params


@traxa_rpc
def taraxa_getWork(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_submitWork(nonce, header_power_hash, mix_digest, **kwargs):
    params = [nonce, header_power_hash, mix_digest]
    return params


@traxa_rpc
def taraxa_submitHashrate(hash_rate, id, **kwargs):
    params = [hash_rate, id]
    return params


@traxa_rpc
def taraxa_getProof(address, storage_keys, tag, **kwargs):
    tag = tag_check(tag)
    params = [address, storage_keys, tag]
    return params


#=============
#below is only taraxa rpc, not in eth
#=============


@traxa_rpc
def taraxa_getStorageRoot(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_flush(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_newFilterEx(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getFilterChangesEx(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getFilterLogsEx(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_getLogsEx(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_register(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_unregister(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_fetchQueuedTransactions(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_signTransaction(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_inspectTransaction(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_notePassword(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_chainId(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_signTransaction(address, data, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, data]
    return params


@traxa_rpc
def taraxa_getDagBlockByHash(hash, fullTransactions=False, **kwargs):
    params = [hash, fullTransactions]
    return params


@traxa_rpc
def taraxa_getDagBlockByLevel(tag, fullTransactions=False, **kwargs):
    tag = tag_check(tag)
    params = [tag, fullTransactions]
    return params


@traxa_rpc
def taraxa_dagBlockLevel(**kwargs):
    params = []
    return params


@traxa_rpc
def taraxa_dagBlockPeriod(**kwargs):
    params = []
    return params


from ..jsonrpc import set, reset
from .methods import syncing
from .methods import coinbase
from .methods import mining
from .methods import hashrate
from .methods import gasPrice
from .methods import accounts
from .methods import getBalance
from .methods import blockNumber
from .methods import getStorageAt
from .methods import getTransactionCount
from .methods import getBlockTransactionCountByHash
from .methods import getBlockTransactionCountByNumber
from .methods import getUncleCountByBlockHash
from .methods import getUncleCountByBlockNumber
from .methods import getCode
from .methods import sign
from .methods import sendTransaction
from .methods import sendRawTransaction
from .methods import call
from .methods import estimateGas
from .methods import getBlockByHash
from .methods import getBlockByNumber
from .methods import getTransactionByHash
from .methods import getTransactionByBlockHashAndIndex
from .methods import getTransactionByBlockNumberAndIndex
from .methods import getTransactionReceipt
from .methods import pendingTransactions
from .methods import getUncleByBlockHashAndIndex
from .methods import getUncleByBlockNumberAndIndex
from .methods import newFilter
from .methods import newBlockFilter
from .methods import newPendingTransactionFilter
from .methods import uninstallFilter
from .methods import getFilterChanges
from .methods import getFilterLogs
from .methods import getLogs
from .methods import getWork
from .methods import submitWork
from .methods import submitHashrate
from .methods import getProof
# from .methods import getDagBlockByLevel
# from .methods import getDagBlockByHash


import json
import asyncio
import websockets
from . import NODE_WS_IP, NODE_WS_PORT, JSONRPC, ID
import logging


class Config():

    def __init__(self, ip=NODE_WS_IP, port=NODE_WS_PORT, jsonrpc=JSONRPC, id=ID):
        self.ip = ip
        self.port = port
        self.jsonrpc = jsonrpc
        self.id = id

    def __str__(self):
        return "ip:{},port:{},jsonrpc:{},id:{}".format(self.ip, self.port, self.jsonrpc, self.id)


config = Config()


def set_ws(params):
    if "ip" in params:
        config.ip = params["ip"]
    if "port" in params:
        config.port = params["port"]
    if "jsonrpc" in params:
        config.port = params["jsonrpc"]
    if "id" in params:
        config.port = params["id"]


def reset_ws():
    config.ip = NODE_WS_IP
    config.port = NODE_WS_PORT
    config.jsonrpc = JSONRPC
    config.id = ID


async def send_ws(data, callback, ip="", port=0):
    if not ip:
        ip = config.ip
    if not port:
        port = config.port

    while True:
        try:
            async with websockets.connect('ws://' + ip + ':' + str(port)) as ws:
                await ws.send(data)
                try:
                    async for msg in ws:
                        _data = json.loads(msg)
                        callback(_data)
                except ConnectionRefusedError as e:
                    logging.info('lose connection,reconnect %s second later.\n%s' % (0.5, data))
                    await asyncio.sleep(0.5)
        except ConnectionRefusedError as e:
            logging.info('connect fail,try again %s second later.\n%s' % (0.5, data))
            await asyncio.sleep(0.5)


def message_ws(jsonrp, method, params, id):
    trx = {"jsonrpc": jsonrp, "method": method, "params": params, "id": id}
    return json.dumps(trx)


def traxa_rpc_ws(func):

    def wrap_func(*args, **kwargs):
        jsonrpc = kwargs.get("jsonrpc", config.jsonrpc)  #默认参数
        method = func.__name__  #jsonrpc方法名同函数名
        params, callback = func(*args, **kwargs)
        id = kwargs.get("id", config.id)  #默认参数
        ip = kwargs.get("ip", config.ip)
        port = kwargs.get("port", config.port)
        trx = message_ws(jsonrpc, method, params, id)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(send_ws(trx, callback, ip=ip, port=port))

    return wrap_func


@traxa_rpc_ws
def eth_subscribe(action, callback, **kwargs):
    params = [action]
    return params, callback


def newOrderedBlock(callback, **kwargs):
    eth_subscribe("newOrderedBlock", callback, **kwargs)


def newDagBlocks(callback, **kwargs):
    eth_subscribe("newDagBlocks", callback, **kwargs)


def newScheduleBlocks(callback, **kwargs):
    eth_subscribe("newScheduleBlocks", callback, **kwargs)


def newDagBlocksFinalized(callback, **kwargs):
    eth_subscribe("newDagBlocksFinalized", callback, **kwargs)


if __name__ == "__main__":
    callback = print
    newDagBlocksFinalized(callback)



import json
import requests

BOOT_NODE_SK = "3800b2875669d9b2053c1aff9224ecfdc411423aac5b5a73d7a45ced1c3b9dcd"
BOOT_NODE_PK = "7b1fcf0ec1078320117b96e9e9ad9032c06d030cf4024a598347a4623a14a421d4f" \
               "030cf25ef368ab394a45e920e14b57a259a09c41767dd50d1da27b627412a"
BOOT_NODE_ADDR = "de2b1203d72d3549ee2f733b00b2789414c7cea5"


def rpc(node_port, data):
    node_name = "127.0.0.1"
    reply = requests.post("http://{}:{}".format(node_name, node_port), data=json.dumps(data))
    if reply is None or reply.status_code != 200 or "error" in reply.text:
        print("Send rpc failed ...", reply.text)
        return
    json_reply = json.loads(reply.text)
    # print("JSON REPLY ", json_reply)
    return json_reply


def taraxa_rpc_get_transaction_count(node_port):
    request = {
        "jsonrpc": "2.0",
        "id": node_port,
        "method": "get_transaction_count",
        "params": [{}]
    }
    json_reply = rpc(node_port, request)
    count = json_reply["result"]["value"]
    return count


def taraxa_rpc_get_dag_size(node_port):
    request = {"jsonrpc": "2.0", "id": node_port, "method": "get_dag_size", "params": [{}]}
    json_reply = rpc(node_port, request)
    dag_size = json_reply["result"]["value"].split(",")
    # print(("DAG size:", dag_size[1]))
    return dag_size


def taraxa_rpc_get_peer_count(node_port):
    request = {"jsonrpc": "2.0", "id": node_port, "method": "get_peer_count", "params": [{}]}
    json_reply = rpc(node_port, request)
    peer = json_reply["result"]["value"]
    print("Node ", node_port, "has", peer, "peers")
    return int(peer)


def taraxa_rpc_get_account_balance(node_port, account_address):
    request = {
        "jsonrpc": "2.0",
        "id": "0",
        "method": "get_account_balance",
        "params": [{
            "address": account_address,
        }]
    }
    json_reply = rpc(node_port, request)
    result = json_reply["result"]
    balance = result["value"]
    found = result["found"]
    # print("Account: ", address,", Balance: ", balance, "Found: ", found)
    return int(balance)


def taraxa_rpc_send_coins(node_port, receiver, value):
    request = {
        "jsonrpc": "2.0",
        "id": "1",
        "method": "send_coin_transaction",
        "params": [{
            "nonce": 0,
            "value": value,
            "receiver": receiver,
            "secret": BOOT_NODE_SK,
        }]
    }
    json_reply = rpc(node_port, request)
    print("Boot node send", value, "coins to", receiver)


def taraxa_rpc_send_many_trx_to_neighbor(node_port, neighbor, number_of_trx_created):
    request = {
        "jsonrpc":
            "2.0",
        "id":
            0,
        "method":
            "create_test_coin_transactions",
        "params": [{
            "delay": 1,
            "number": int(number_of_trx_created),
            "nonce": 0,
            "receiver": neighbor
        }]
    }
    json_reply = rpc(node_port, request)
    # print("Node", node_port, "send",
    # number_of_trx_created, "trxs to", neighbor)


def taraxa_rpc_get_executed_trx_count(node_port):
    request = {
        "jsonrpc": "2.0",
        "id": node_port,
        "method": "get_executed_trx_count",
        "params": [{}]
    }
    json_reply = rpc(node_port, request)
    result = json_reply["result"]
    count = result["value"]
    return int(count)


def taraxa_rpc_get_executed_blk_count(node_port):
    request = {
        "jsonrpc": "2.0",
        "id": node_port,
        "method": "get_executed_blk_count",
        "params": [{}]
    }
    json_reply = rpc(node_port, request)
    result = json_reply["result"]
    count = result["value"]
    return int(count)


# Below are taraxa rpc


def taraxa_rpc_blockNumber(node_port):
    request = {"jsonrpc": "2.0", "id": node_port, "method": "taraxa_blockNumber", "params": []}
    json_reply = rpc(node_port, request)
    block_num = json_reply["result"]
    return int(block_num, 0)


from .utl import *
from ._http import *


# TODO
@traxa_rpc
def web3_clientVersion(**kwargs):
    params = []
    return params


# TODO
@traxa_rpc
def web3_sha3(data, **kwargs):
    params = [data]
    return params


def hex2int(hex):
    if hex == '':
        return -1
    if type(hex) == int:
        return hex
    if type(hex) == str:
        return int(hex, 16)
    return hex


def tag_check(tag):
    # tag data type:"latest",'10','0xa',10,0xa
    if type(tag) == str:
        try:
            tag = hex(int(tag))
        except ValueError:
            try:
                tag = hex(int(tag, 16))  #16进制字符'0xa'
            except:
                tag = tag

    elif type(tag) == int:
        tag = hex(int(tag))

    return tag



from .utl import *
from ._http import *


@traxa_rpc
def eth_syncing():
    params = []
    return params


@traxa_rpc
def eth_coinbase(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_mining():
    params = []
    return params


@traxa_rpc
def eth_hashrate(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_gasPrice(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_accounts(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getBalance(address, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


@traxa_rpc
def eth_blockNumber(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getStorageAt(address, position, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, position, tag]
    return params


@traxa_rpc
def eth_getTransactionCount(address, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


@traxa_rpc
def eth_getBlockTransactionCountByHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def eth_getBlockTransactionCountByNumber(tag, **kwargs):
    tag = tag_check(tag)
    params = [tag]
    return params


@traxa_rpc
def eth_getUncleCountByBlockHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def eth_getUncleCountByBlockNumber(tag, **kwargs):
    tag = tag_check(tag)
    params = [tag]
    return params


@traxa_rpc
def eth_getCode(address, tag, **kwargs):
    tag = tag_check(tag)
    params = [address, tag]
    return params


# TODO
@traxa_rpc
def eth_sign(address, data, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, data]
    return params


# TODO
@traxa_rpc
def eth_sendTransaction(trx, **kwargs):
    params = [trx]
    return params


@traxa_rpc
def eth_sendRawTransaction(trx, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [trx]
    return params


# TODO
@traxa_rpc
def eth_call(trx, tag, **kwargs):
    tag = tag_check(tag)
    params = [
        {
            # "from": trx['from'],
            # "to": trx['to'],
            # "gas": trx['gas'],
            # "gasPrice": trx['gasPrice'],
            # "value": trx['value'],
            # "data": trx['data'],
        },
        tag
    ]
    return params


# TODO
@traxa_rpc
def eth_estimateGas(trx, tag, **kwargs):
    tag = tag_check(tag)
    params = [
        {
            # "from": trx['from'],
            # "to": trx['to'],
            # "gas": trx['gas'],
            # "gasPrice": trx['gasPrice'],
            # "value": trx['value'],
            # "data": trx['data'],
        },
        tag
    ]
    return params


@traxa_rpc
def eth_getBlockByHash(hash, fullTransactions=False, **kwargs):
    params = [hash, fullTransactions]
    return params


@traxa_rpc
def eth_getBlockByNumber(tag, fullTransactions=False, **kwargs):
    tag = tag_check(tag)
    params = [tag, fullTransactions]
    return params


@traxa_rpc
def eth_getTransactionByHash(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def eth_getTransactionByBlockHashAndIndex(hash, index, **kwargs):
    params = [hash, index]
    return params


@traxa_rpc
def eth_getTransactionByBlockNumberAndIndex(tag, index, **kwargs):
    tag = tag_check(tag)
    params = [tag, index]
    return params


@traxa_rpc
def eth_getTransactionReceipt(hash, **kwargs):
    params = [hash]
    return params


@traxa_rpc
def eth_pendingTransactions(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getUncleByBlockHashAndIndex(hash, index, **kwargs):
    params = [hash, index]
    return params


@traxa_rpc
def eth_getUncleByBlockNumberAndIndex(tag, index, **kwargs):
    tag = tag_check(tag)
    params = [tag, index]
    return params


@traxa_rpc
def eth_newFilter(filter, **kwargs):
    params = [filter]
    return params


@traxa_rpc
def eth_newBlockFilter(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_newPendingTransactionFilter(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_uninstallFilter(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def eth_getFilterChanges(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def eth_getFilterLogs(id, **kwargs):
    params = [id]
    return params


@traxa_rpc
def eth_getLogs(filter, **kwargs):
    params = [filter]
    return params


@traxa_rpc
def eth_getWork(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_submitWork(nonce, header_power_hash, mix_digest, **kwargs):
    params = [nonce, header_power_hash, mix_digest]
    return params


@traxa_rpc
def eth_submitHashrate(hash_rate, id, **kwargs):
    params = [hash_rate, id]
    return params


@traxa_rpc
def eth_getProof(address, storage_keys, tag, **kwargs):
    tag = tag_check(tag)
    params = [address, storage_keys, tag]
    return params


#=============
#below is only taraxa rpc, not in eth
#=============


@traxa_rpc
def eth_getStorageRoot(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_flush(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_newFilterEx(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getFilterChangesEx(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getFilterLogsEx(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_getLogsEx(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_register(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_unregister(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_fetchQueuedTransactions(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_signTransaction(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_inspectTransaction(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_notePassword(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_chainId(**kwargs):
    params = []
    return params


@traxa_rpc
def eth_signTransaction(address, data, tag="latest", **kwargs):
    tag = tag_check(tag)
    params = [address, data]
    return params


@traxa_rpc
def eth_getDagBlockByHash(hash, fullTransactions=False, **kwargs):
    params = [hash, fullTransactions]
    return params


@traxa_rpc
def eth_getDagBlockByLevel(tag, fullTransactions=False, **kwargs):
    tag = tag_check(tag)
    params = [tag, fullTransactions]
    return params


if __name__ == "__main__":
    address = [
        "de2b1203d72d3549ee2f733b00b2789414c7cea5", "973ecb1c08c8eb5a7eaa0d3fd3aab7924f2838b0",
        "4fae949ac2b72960fbe857b56532e2d3c8418d5e", "415cf514eb6a5a8bd4d325d4874eae8cf26bcfe0",
        "b770f7a99d0b7ad9adf6520be77ca20ee99b0858"
    ]
    # r = eth_getBlockByHash('0x1571a0204e280d854d1c26821f4a77936745a9d9b869fcf7f18d3f6db74d42ce',
    #                        True)
    # print(r['result'])
    #trx = {'a': 1}
    #r = eth_sendTransaction(trx, ip='333.333')
    tags = ["latest", '10', '0xa', 10, 0xa]
    for t in tags:
        print(tag_check(t))



from .. import eth


class Web3:

    def __init__(self, ip, port):
        self.ip = ip
        self.port = port

    def syncing(self, **kwargs):

        eth.syncing(ip=self.ip, port=self.port, **kwargs)

    def coinbase(self, **kwargs):

        eth.coinbase(ip=self.ip, port=self.port, **kwargs)

    def mining(self, **kwargs):

        eth.mining(ip=self.ip, port=self.port, **kwargs)

    def hashrate(self, **kwargs):

        eth.hashrate(ip=self.ip, port=self.port, **kwargs)

    def gasPrice(self, **kwargs):

        eth.gasPrice(ip=self.ip, port=self.port, **kwargs)

    def accounts(self, **kwargs):

        eth.accounts(ip=self.ip, port=self.port, **kwargs)

    def getBalance(self, address, tag="latest", **kwargs):

        eth.getBalance(address, tag="latest", ip=self.ip, port=self.port, **kwargs)

    def blockNumber(self, **kwargs):

        eth.blockNumber(ip=self.ip, port=self.port, **kwargs)

    def getStorageAt(self, address, position, tag="latest", **kwargs):

        eth.getStorageAt(address, position, tag="latest", ip=self.ip, port=self.port, **kwargs)

    def getTransactionCount(self, address, tag="latest", **kwargs):

        eth.getTransactionCount(address, tag="latest", ip=self.ip, port=self.port, **kwargs)

    def getBlockTransactionCountByHash(self, hash, **kwargs):

        eth.getBlockTransactionCountByHash(hash, ip=self.ip, port=self.port, **kwargs)

    def getBlockTransactionCountByNumber(self, tag, **kwargs):

        eth.getBlockTransactionCountByNumber(tag, ip=self.ip, port=self.port, **kwargs)

    def getUncleCountByBlockHash(self, hash, **kwargs):

        eth.getUncleCountByBlockHash(hash, ip=self.ip, port=self.port, **kwargs)

    def getUncleCountByBlockNumber(self, tag, **kwargs):

        eth.getUncleCountByBlockNumber(tag, ip=self.ip, port=self.port, **kwargs)

    def getCode(self, address, tag, **kwargs):

        eth.getCode(address, tag, ip=self.ip, port=self.port, **kwargs)

    def sign(self, address, data, tag="latest", **kwargs):

        eth.sign(address, data, tag="latest", ip=self.ip, port=self.port, **kwargs)

    def sendTransaction(self, trx, **kwargs):

        eth.sendTransaction(trx, ip=self.ip, port=self.port, **kwargs)

    def sendRawTransaction(self, trx, **kwargs):

        eth.sendRawTransaction(trx, ip=self.ip, port=self.port, **kwargs)

    def call(self, trx, tag, **kwargs):

        eth.call(trx, tag, ip=self.ip, port=self.port, **kwargs)

    def estimateGas(self, trx, tag, **kwargs):

        eth.estimateGas(trx, tag, ip=self.ip, port=self.port, **kwargs)

    def getBlockByHash(self, hash, fullTransactions=False, **kwargs):

        eth.getBlockByHash(hash, fullTransactions=False, ip=self.ip, port=self.port, **kwargs)

    def getBlockByNumber(self, tag, fullTransactions=False, **kwargs):

        eth.getBlockByNumber(tag, fullTransactions=False, ip=self.ip, port=self.port, **kwargs)

    def getTransactionByHash(self, hash, **kwargs):

        eth.getTransactionByHash(hash, ip=self.ip, port=self.port, **kwargs)

    def getTransactionByBlockHashAndIndex(self, hash, index, **kwargs):

        eth.getTransactionByBlockHashAndIndex(hash, index, ip=self.ip, port=self.port, **kwargs)

    def getTransactionByBlockNumberAndIndex(self, tag, index, **kwargs):

        eth.getTransactionByBlockNumberAndIndex(tag, index, ip=self.ip, port=self.port, **kwargs)

    def getTransactionReceipt(self, hash, **kwargs):

        eth.getTransactionReceipt(hash, ip=self.ip, port=self.port, **kwargs)

    def pendingTransactions(self, **kwargs):

        eth.pendingTransactions(ip=self.ip, port=self.port, **kwargs)

    def getUncleByBlockHashAndIndex(self, hash, index, **kwargs):

        eth.getUncleByBlockHashAndIndex(hash, index, ip=self.ip, port=self.port, **kwargs)

    def getUncleByBlockNumberAndIndex(self, tag, index, **kwargs):

        eth.getUncleByBlockNumberAndIndex(tag, index, ip=self.ip, port=self.port, **kwargs)

    def newFilter(self, filter, **kwargs):

        eth.newFilter(filter, ip=self.ip, port=self.port, **kwargs)

    def newBlockFilter(self, **kwargs):

        eth.newBlockFilter(ip=self.ip, port=self.port, **kwargs)

    def newPendingTransactionFilter(self, **kwargs):

        eth.newPendingTransactionFilter(ip=self.ip, port=self.port, **kwargs)

    def uninstallFilter(self, id, **kwargs):

        eth.uninstallFilter(id, ip=self.ip, port=self.port, **kwargs)



NODE_IP = "0.0.0.0"
NODE_PORT = 7777
NODE_WS_IP = "0.0.0.0"
NODE_WS_PORT = 8777
JSONRPC = "2.0"
ID = 1

from .utl import tag_check

from .http_eth import *
from .http_taraxa import *
from .http_web3 import *
from .http_net import *

from ._http import send, message, traxa_rpc, set, reset
from ._websocket import send_ws, message_ws, traxa_rpc_ws, set_ws, reset_ws
from ._websocket import eth_subscribe, newDagBlocks, newScheduleBlocks, newDagBlocksFinalized



from ..jsonrpc import set, reset
from .methods import dagBlockLevel
from .methods import dagBlockPeriod
from .methods import getDagBlockByHash
from .methods import getDagBlockByLevel
from .methods import blockNumber


from ..jsonrpc.http_taraxa import *


def dag_block_hex2int(block):
    keys = ['level', 'period', 'number', 'timestamp']
    for key in keys:
        block[key] = hex2int(block[key])
    return block


def getDagBlockByHash(hash, fullTransactions=False, **kwargs):
    r = taraxa_getDagBlockByHash(hash, fullTransactions, **kwargs)
    block = r['result']
    if 'result' in r:
        block = r['result']
        if block:
            block = dag_block_hex2int(block)
        return block
    else:
        raise Exception(r["error"])


def getDagBlockByLevel(tag, fullTransactions=False, **kwargs):
    r = taraxa_getDagBlockByLevel(tag, fullTransactions, **kwargs)
    if 'result' in r:
        blocks = r['result']
        blocks = list(map(lambda block: dag_block_hex2int(block), blocks))
        return blocks
    else:
        raise Exception(r["error"])


def dagBlockLevel(**kwargs):
    r = taraxa_dagBlockLevel(**kwargs)
    level = hex2int(r['result'])
    return level


def dagBlockPeriod(**kwargs):
    r = taraxa_dagBlockPeriod(**kwargs)
    period = hex2int(r['result'])
    return period


def blockNumber(**kwargs):
    r = taraxa_blockNumber(**kwargs)
    number = hex2int(r['result'])
    return number


from . import NODE_IP, NODE_PORT, JSONRPC, ID
import requests
import json
import logging


class Config():

    def __init__(self, ip=NODE_IP, port=NODE_PORT, jsonrpc=JSONRPC, id=ID):
        self.ip = ip
        self.port = port
        self.jsonrpc = jsonrpc
        self.id = id

    def __str__(self):
        return "ip:{},port:{},jsonrpc:{},id:{}".format(self.ip, self.port, self.jsonrpc, self.id)


config = Config()


def set(params):
    if "ip" in params:
        config.ip = params["ip"]
    if "port" in params:
        config.port = params["port"]
    if "jsonrpc" in params:
        config.port = params["jsonrpc"]
    if "id" in params:
        config.port = params["id"]


def reset():
    config.ip = NODE_IP
    config.port = NODE_PORT
    config.jsonrpc = JSONRPC
    config.id = ID


def send(data, ip="", port=0):
    if not ip:
        ip = config.ip
    if not port:
        port = config.port
    if type(data) == dict:
        data = json.dumps(data)
    elif type(data) == str:
        pass
    else:
        raise Exception('send data must be json string or dict')

    request = requests.post("http://{}:{}".format(ip, port), data=data)
    return request


def message(jsonrpc, method, params, id):
    trx = {"jsonrpc": jsonrpc, "method": method, "params": params, "id": id}
    return json.dumps(trx)


def traxa_rpc(func):

    def wrap_func(*args, **kwargs):
        #要提交的参数
        jsonrpc = kwargs.get("jsonrpc", config.jsonrpc)  #默认参数
        method = func.__name__  #jsonrpc方法名同函数名
        params = func(*args, **kwargs)
        id = kwargs.get("id", config.id)  #默认参数
        msg = message(jsonrpc, method, params, id)

        #要提交的节点
        ip = kwargs.get("ip", config.ip)
        port = kwargs.get("port", config.port)
        r = send(msg, ip, port)
        return r.json()

    return wrap_func


from ..jsonrpc.http_eth import *


def syncing(**kwargs):
    r = eth_syncing(**kwargs)
    return r['result']


def coinbase(**kwargs):
    r = eth_coinbase(**kwargs)
    return r['result']


def mining(**kwargs):
    r = eth_mining(**kwargs)
    return r['result']


def hashrate(**kwargs):
    r = eth_hashrate(**kwargs)
    number = hex2int(r['result'])
    return number


#TODO
def gasPrice(**kwargs):
    r = eth_gasPrice(**kwargs)
    return r


def accounts(**kwargs):
    r = eth_accounts(**kwargs)
    return r['result']


def getBalance(address, tag="latest", **kwargs):
    r = eth_getBalance(address, tag)
    number = hex2int(r['result'])
    return number


def blockNumber(**kwargs):
    r = eth_blockNumber(**kwargs)
    number = hex2int(r['result'])
    return number


#no in taraxa rpc
def getStorageAt(address, position, tag="latest", **kwargs):
    r = eth_getStorageAt(**kwargs)
    return r


#no in taraxa rpc
def getTransactionCount(address, tag="latest", **kwargs):
    r = eth_getTransactionCount(address, tag, **kwargs)
    number = hex2int(r['result'])
    return number


#no in taraxa rpc
def getBlockTransactionCountByHash(hash, **kwargs):
    r = eth_getTransactionCount(hash, **kwargs)
    number = hex2int(r['result'])
    return number


def getBlockTransactionCountByNumber(tag="latest", **kwargs):
    r = eth_getBlockTransactionCountByNumber(tag, **kwargs)
    return r['result']


def getUncleCountByBlockHash(hash, **kwargs):
    r = eth_getUncleCountByBlockHash(hash, **kwargs)
    return r['result']


def getUncleCountByBlockNumber(tag="latest", **kwargs):
    r = eth_getUncleCountByBlockNumber(tag, **kwargs)
    return r['result']


def getCode(address, tag="latest", **kwargs):
    r = eth_getCode(address, tag, **kwargs)
    return r['result']


def sign(address, data, tag="latest", **kwargs):
    r = eth_sign(address, data, tag, **kwargs)
    return r['result']


def sendTransaction(trx, **kwargs):
    r = eth_sendTransaction(trx, **kwargs)
    return r['result']


def sendRawTransaction(trx, **kwargs):
    r = eth_sendRawTransaction(trx, **kwargs)
    return r['result']


def call(trx, tag="latest", **kwargs):
    r = eth_call(trx, **kwargs)
    return r


def estimateGas(trx, tag="latest", **kwargs):
    r = eth_estimateGas(trx, tag, **kwargs)
    return r


def getBlockByHash(hash, fullTransactions=False, **kwargs):
    r = eth_getBlockByHash(hash, fullTransactions, **kwargs)
    block = r['result']
    if not block:
        return block
    block_key_number = [
        'difficulty', 'gasUsed', 'nonce', 'number', 'size', 'timestamp', 'totalDifficulty'
    ]
    for key in block_key_number:
        block[key] = hex2int(block[key])
    return block


def getBlockByNumber(tag, fullTransactions=False, **kwargs):
    r = eth_getBlockByNumber(tag, fullTransactions, **kwargs)
    block = r['result']
    if not block:
        return block
    block_key_number = [
        'difficulty', 'gasUsed', 'nonce', 'number', 'size', 'timestamp', 'totalDifficulty'
    ]
    for key in block_key_number:
        block[key] = hex2int(block[key])
    return block


def getTransactionByHash(hash, **kwargs):
    r = eth_getTransactionByHash(hash, **kwargs)
    return r


def getTransactionByBlockHashAndIndex(hash, index, **kwargs):
    r = eth_getTransactionByBlockHashAndIndex(hash, index, **kwargs)
    return r


def getTransactionByBlockNumberAndIndex(tag, index, **kwargs):
    r = eth_getTransactionByBlockNumberAndIndex(tag, index, **kwargs)
    return r


def getTransactionReceipt(hash, **kwargs):
    r = eth_getTransactionReceipt(hash)
    return r


def pendingTransactions(**kwargs):
    r = eth_pendingTransactions(**kwargs)
    return r['result']


def getUncleByBlockHashAndIndex(hash, index, **kwargs):
    r = eth_getUncleByBlockHashAndIndex(hash, index, **kwargs)
    return r['result']


def getUncleByBlockNumberAndIndex(tag, index, **kwargs):
    r = eth_getUncleByBlockNumberAndIndex(tag, index, **kwargs)
    return r['result']


def newFilter(filter, **kwargs):
    r = eth_newFilter(filter, **kwargs)
    return r['result']


def newBlockFilter(**kwargs):
    r = eth_newBlockFilter(**kwargs)
    return r['result']


def newPendingTransactionFilter(**kwargs):
    r = eth_newPendingTransactionFilter(**kwargs)
    return r['result']


def uninstallFilter(id, **kwargs):
    r = eth_uninstallFilter(id, **kwargs)
    return r['result']


def getFilterChanges(id, **kwargs):
    r = eth_getFilterChanges(id, **kwargs)
    return r['result']


def getFilterLogs(id, **kwargs):
    r = eth_getFilterLogs(id, **kwargs)
    return r['result']


def getLogs(filter, **kwargs):
    r = eth_getLogs(filter, **kwargs)
    return r['result']


def getWork(**kwargs):
    r = eth_getWork(**kwargs)
    return r['result']


def submitWork(nonce, header_power_hash, mix_digest, **kwargs):
    r = eth_submitWork(nonce, header_power_hash, mix_digest, **kwargs)
    return r['result']


def submitHashrate(hash_rate, id, **kwargs):
    r = eth_submitHashrate(hash_rate, id, **kwargs)
    return r['result']


def getProof(address, storage_keys, tag, **kwargs):
    r = eth_getProof(address, storage_keys, tag, **kwargs)
    return r['result']


if __name__ == "__main__":
    blockNumber()



import json


def create_taraxa_conf(path_fn, num_conf, secrets, boot_node_pk, boot_node_addr):
    for i in range(num_conf):
        conf = {
            "node_secret": secrets[i],
            "db_path": "/tmp/taraxa" + str(i),
            "dag_processing_threads": 1,
            "network_address": "127.0.0.1",
            "network_listen_port": 10002 + i,
            "network_simulated_delay": 0,
            "network_transaction_interval": 100,
            "network_bandwidth": 40,
            "network_boot_nodes": [{
                "id": boot_node_pk,
                "ip": "127.0.0.1",
                "port": 10002
            }],
            "network_id": "testnet",
            "rpc_port": 7777 + i,
            "ws_port": 8777 + i,
            "test_params": {
                "block_proposer": [0, 1, 100, 1000],
                "pbft": [2000, 1, 1000000000, 6000, 0]
            },
            "genesis_state": {
                "account_start_nonce": 0,
                "block": {
                    "level":
                        0,
                    "tips": [],
                    "trxs": [],
                    "sig":
                        "b7e22d46c1ba94d5e8347b01d137b5c428fcbbeaf0a77fb024cbbf1517656ff00d04f7f25be608c321b0d7483c402c294ff46c49b265305d046a52236c0a363701",
                    "hash":
                        "c9524784c4bf29e6facdd94ef7d214b9f512cdfd0f68184432dab85d053cbc69",
                    "sender":
                        "de2b1203d72d3549ee2f733b00b2789414c7cea5",
                    "pivot":
                        "0000000000000000000000000000000000000000000000000000000000000000",
                    "timestamp":
                        1564617600
                },
                "accounts": {
                    boot_node_addr: {
                        "balance": 9007199254740991
                    }
                }
            }
        }
        with open(path_fn(i), "w") as f:
            f.write(json.dumps(conf, indent=2))


from .utl import *
from ._http import *


@traxa_rpc
def net_version(**kwargs):
    params = []
    return params


@traxa_rpc
def net_peerCount(**kwargs):
    params = []
    return params


@traxa_rpc
def net_listening(**kwargs):
    params = []
    return params